<?php

class File {
  function __construct($path) {
    $this->path = $path;
  }
  function name() {
    $pathParts = explode('/', $this->path);
    return array_pop($pathParts) . ($this->is_dir() ? '/' : '');
  }

  function contents() { return file_get_contents($this->path); }
  function is_dir() { return is_dir($this->path); }
}

class Dir {
  function __construct($path) {
    $this->path = $path;
  }
  function files() {
    $filenames = $foldernames = array();
    $handle = opendir($this->path);
    while (false !== ($file = readdir($handle))) {
      if ($file[0] != '.') {
        if (is_dir($this->path . '/' . $file)) $foldernames[] = $file;
        else $filenames[] = $file;
      }
    }
    natcasesort($foldernames);
    natcasesort($filenames);
    $filenames = array_merge($foldernames, $filenames);
    $files = array();
    foreach ($filenames as $file)
      $files[] = new File($this->path . '/' . $file);
    closedir($handle);
    return $files;
  }
}

// helpers
function url_parts($path) {
  $parts = explode('/', $path);
  $return = array();
  foreach ($parts as $name)
    if ($name)
      $return[] = $name;
  return $return;
}
function fancyPath($folders) {
  $fancyTitle = '';
  $link = '/';
  foreach ($folders as $folder) {
    $link .= $folder . '/';
    $fancyTitle .= '/<a href="'.h($link).'" class="top">'.h($folder).'</a>';
  }
  return $fancyTitle;
}
function h($t) {
	return htmlspecialchars($t);
}
function startswith($hay, $needle) {
	return $needle === $hay or strpos($hay, $needle) === 0;
}
function endswith($hay, $needle) {
    return $needle === $hay or strpos(strrev($hay), strrev($needle)) === 0;
}

$dir = false;
$file = false;

$page = urldecode($_SERVER['REQUEST_URI']);
$urlParts = url_parts($page);
$rootLink = empty($urlParts) ? h($_SERVER['HTTP_HOST']) : '<a href="/" class="top">'.h($_SERVER['HTTP_HOST']).'</a>';
$pageName = array_pop($urlParts);
$path = $_SERVER['DOCUMENT_ROOT'];

// file
if ($pageName[0] == '?') {
  $pageName = substr($pageName, 1);
  $path .= implode('/', $urlParts) . '/' . $pageName;
  $file = is_file($path) ? new File($path) : false;
// dir
} else {
  $path .= $page;
  $dir = is_dir($path) ? new Dir($path) : false;
}

?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
  <meta http-equiv="Content-type" content="text/html; charset=utf-8" />
  <meta name="viewport" content="width=device-width; initial-scale=1.0; maximum-scale=1.0; user-scalable=0;" />
  <title><?php echo h($pageName) ?></title>
  <link rel="stylesheet" href="/.dirlist/iphonedirlist.css" type="text/css" />
  </style>
</head>
<body>
<h1><?php echo $rootLink.fancyPath($urlParts) . '/ ' . h($pageName); ?></h1>

<?php if ($dir) { ?>

<ul>
<?php foreach ($dir->files() as $file) : ?>
    <a href="<?php echo h($file->name()) ?>"><li>
    	<?php echo h($file->name()) ?></li></a>
<?php endforeach; ?>
  </a>

<?php } else if ($file !== false) { ?>

<pre><?php echo h($file->contents()) ?></pre>

<?php } else { ?>

<p>Errorrrr</p>

<?php } ?>

</body>
</html>